%%jtms.pl
%%There are no user commands in this file

%%jtms_create_node(+Head, +Justifications)
%%creates a new node for Head justified by Justifications if node doesnt exist
%%updates the existing node with new Justifications otherwise
jtms_create_node(Head, J):-	jtms_justify(J),
							node(Head, JList, In), 
							retract(node(Head, JList, In)), 
							assert(node(Head, [J|JList], true)).
jtms_create_node(Head, J):-	node(Head, JList, In), 
							retract(node(Head, JList, In)), 
							assert(node(Head, [J|JList], In)).
jtms_create_node(Head, J):-	jtms_justify(J), assert(node(Head, [J], true)).
jtms_create_node(Head, J):-	assert(node(Head, [J], false)).

%%jtms_justify(+List)
%%returns true iff at least one of the set of Horns in the list is IN
jtms_justify([]):- fail.
jtms_justify([true|_]).
jtms_justify([H|_]):- node(H, _, true).
jtms_justify([_|T]):- jtms_justify(T).

%%jtms_retract(+Head)
%%retracts nodes which are justified by Head and removes Horns containing Head from
%%the in-list of other nodes
jtms_retract(Head):-	findall(node(X,Y,Z), (node(X,Y,Z), inner_member(Head,Y)), []),
						write('The JTMS found the following no nodes which may be justified by the retracted sentence\n').
jtms_retract(Head):-	findall(node(X,Y,Z), (node(X,Y,Z), inner_member(Head,Y)), Bag),
						write('The JTMS found the following nodes which may be justified by the retracted sentence\n'),
						%%So the plan is to remove the justification and its permutations from the IN-LIST
						pprint(Bag),
						jtms_remove_justifications(Head, Bag).

%%jtms_remove_justifications(+Head, +Nodes)
%%Cycles through nodes removing Head from the justification lists and changin node label where appropriate
jtms_remove_justifications(_,[]).
jtms_remove_justifications(Head, [H|T]):-	jtms_remove_justification(Head, H), 
											jtms_remove_justifications(Head, T).

%%jtms_remove_justifications(+Head, +Node)
%%updates In-list of one node, retracts appropriate facts and 
%%recursively checks other affected nodes
jtms_remove_justification(Head, node(Head2, InList, Label)):-	retract(node(Head2, InList, Label)),
																jtms_remove_from_inlist(Head, InList, NewInList),
																jtms_create_node(Head2, NewInList),
																jtms_check_facts(Head2).
%%jtms_check_facts(+Head)
%%If a fact is currently beleived yet has no justifications then
%%retract it then check facts justified by this one
jtms_check_facts(Head):- node(Head, _, true).
jtms_check_facts(Head):-	node(Head, _, false), 
							retract_horn(horn([true], Head)),
							write('Removing horn('), write([true]), write(','), write(Head), write(') from the knowledge base\n').
				
%%jtms_remove_from_inlist(+Head, +InList, -NewInList)
%%removes all horns containing Head from the justification list supplied
jtms_remove_from_inlist(Head, InList, NewInList):- jtms_remove_from_inlist(Head, InList, [], NewInList).
jtms_remove_from_inlist(_, [], A, A).
jtms_remove_from_inlist(Head, [H|T], Sofar, Answer):-	member(Head, H),!,
														jtms_remove_from_inlist(Head, T, Sofar, Answer).
jtms_remove_from_inlist(Head, [H|T], Sofar, Answer):-   jtms_remove_from_inlist(Head, T, [H|Sofar], Answer).